pub struct MaskedSurface<'s, S: AsRef<[P]> + AsMut<[P]>, P: Copy + Clone + Sized + Default> { /* private fields */ }Expand description
A surface with a mask color. Pixels of the mask color will not blit to the destination.
A MaskedSurface can be locked or unlocked. If locked, the surface can’t be mutated, but blitting will be faster.
Implementations
Sourceimpl<'s, S: AsRef<[P]> + AsMut<[P]>, P: Copy + Clone + Sized + Default + Eq + PartialEq> MaskedSurface<'s, S, P>
impl<'s, S: AsRef<[P]> + AsMut<[P]>, P: Copy + Clone + Sized + Default + Eq + PartialEq> MaskedSurface<'s, S, P>
pub const fn new(surface: Surface<'s, S, P>, mask_color: P) -> Self
Sourcepub const fn set_mask_color(&mut self, mask_color: P) -> Result<(), Error>
pub const fn set_mask_color(&mut self, mask_color: P) -> Result<(), Error>
Set the color of the mask. Returns an error if the surface is locked.
Sourcepub fn lock(&mut self)
Available on crate feature std only.
pub fn lock(&mut self)
std only.Lock the surface, optimizing blit speed while preventing pixel manipulation.
Sourcepub const fn is_locked(&self) -> bool
Available on crate feature std only.
pub const fn is_locked(&self) -> bool
std only.Returns true if the surface is locked.
Sourcepub fn unlock(&mut self)
Available on crate feature std only.
pub fn unlock(&mut self)
std only.Unlock the surface. Blit speed will be unoptimized while pixel manipulation will be permitted.
Sourcepub const fn surface(&self) -> &Surface<'s, S, P>
pub const fn surface(&self) -> &Surface<'s, S, P>
Returns a reference of the surface.
Note that self.blit(&mut destination) is not the same as self.surface.blit(&mut destination)
because the latter won’t apply the mask.
Sourcepub const fn surface_mut(&mut self) -> Result<&mut Surface<'s, S, P>, Error>
pub const fn surface_mut(&mut self) -> Result<&mut Surface<'s, S, P>, Error>
Returns a mutable reference of the surface. Returns an error if the masked surface is locked.